(ns com.swir.linux.tm.luarpc.session)

(let [i (atom 0)]
  (defn get-session-id []
    (swap! i inc)))

(let [i (atom (byte 0))] 
  (defn get-cmd-id []
    (swap! i #(byte (mod (inc %) 128)))))

(def token-session (ref {}))
(def cmd-callback (ref {}))
(def token-cmds (ref {}))
(def resp-callback (ref {}))

(defn add-session [prefix session]
  (let [token (str prefix "-" (get-session-id))] 
    (dosync (alter token-session assoc token session))
    token))

(defn remove-session [token clbks-handler ses-handler]
  (let [[clbks ses]
        (dosync
          (let [cmds (get (ensure token-cmds) token)
                clbks (vals (select-keys (ensure cmd-callback) cmds))
                ses (get (ensure token-session) token)]
            (alter token-cmds dissoc token)
            (apply alter cmd-callback dissoc cmds)
            (alter token-session dissoc token)
            [clbks ses]))]
    (clbks-handler clbks)
    (ses-handler ses)))

(defn add-clbk [token clbk]
  (let [id (get-cmd-id)]
    (dosync
      (alter cmd-callback assoc id clbk)
      (alter token-cmds update-in [token] #(if % (conj % id) #{id})))
    id))

(defn remove-clbk [token id clbk-handler]
  (let [clbk 
        (dosync 
          (let [clbk (get (ensure cmd-callback) id)]
            (alter cmd-callback dissoc id)
            (alter token-cmds update-in [token] #(disj % id))
            clbk))]
    (clbk-handler clbk)))

(defn add-resp-clbk [token cmd-name clbk]
  (dosync (alter resp-callback assoc [token cmd-name] clbk)))

(defn remove-resp-clbk [token cmd-name]
  (dosync (alter resp-callback dissoc [token cmd-name])))

; TODO: handle errors (exception during execution, handler not found, ...)
;       multiple results ?
(defn call-resp-clbk [token cmd-name value]
  (let [clbk (@resp-callback [token cmd-name])]
    (if (isa? (class clbk) com.swir.linux.tm.luarpc.ILuaRPCHandler)
      [(.call clbk value)]
      (clbk value))))
